; *****************************************************************************
; *****************************************************************************
;
;       File    : Io.Asm
;       Author  : Paul Robson (autismuk@aol.com)
;       Purpose : Functions for Z80 IN and OUT commands
;       Created : 2nd October 1997
;       Modified: 11th November 1997
;
;       Functions :
;               IInPort         Read port 'AL' from port AL
;               IOutPort        Write value AH to port AL
;               IScanKeys       Scan Keys - update bits and joystick byte
;               ICheckSpecial   Special Key Check
;
; *****************************************************************************
; *****************************************************************************

.386p 
.model flat
include extern.inc
.code

; *****************************************************************************
;
;                  Read from port AH:AL into register AL
;
; *****************************************************************************

IInPort Proc
        test    al,1                    ; even port read
        jnz     _IIPX                   ; if not exit

        push    eax
        mov     al,1                    ; VSYNC on
        call    SCSetSync
        pop     eax

        mov     al,ah                   ; AL is the column being tested.
        mov     ah,255                  ; set the default bits

        test    al,1                    ; row 1 (Shift,Z,X,C,V)
        jnz     _IIP1
        and     ah,[BitTable+0]
_IIP1:  test    al,2                    ; row 2 (A,S,D,F,G)
        jnz     _IIP2
        and     ah,[BitTable+1]
_IIP2:  test    al,4                    ; row 3 (Q,W,E,R,T)
        jnz     _IIP3
        and     ah,[BitTable+2]
_IIP3:  test    al,8                    ; row 4 (1,2,3,4,5)
        jnz     _IIP4
        and     ah,[BitTable+3]
_IIP4:  test    al,16                   ; row 5 (0,9,8,7,6)
        jnz     _IIP5
        and     ah,[BitTable+4]
_IIP5:  test    al,32                   ; row 6 (P,O,I,Y,U)
        jnz     _IIP6
        and     ah,[BitTable+5]
_IIP6:  test    al,64                   ; row 7 (Enter,L,K,J,H)
        jnz     _IIP7
        and     ah,[BitTable+6]
_IIP7:  test    al,128                  ; row 8 (Space,Full Stop,M,N,B)
        jnz     _IIP8
        and     ah,[BitTable+7]
_IIP8:
        and     ah,01Fh                 ; clear bit 7,6 and 5
        or      ah,060h                 ; set bits 5 for PAL
        mov     al,ah                   ; return given value.
        ret
_IIPX:  mov     al,255                  ; return 255
        ret

EndP IInPort

BitTable db     0,0,0,0,0,0,0,0         ; table for pressed key flags

; *****************************************************************************
;
;                        Scan Keys, updating the key tables
;
; *****************************************************************************

IScanKeys Proc
        push    eax
        push    ebx
        push    ecx
        push    edx
        push    ebp

        mov     ecx,offset BitTable     ; ECX points to the data
        mov     ebp,offset ScanCodeTable; EBP points to the scan codes
        mov     dl,8                    ; DL counts the columns

ISK1:   mov     dh,255                  ; set to all 1's
        mov     [ecx],dh
        mov     dh,1                    ; the initial mask 00000001

ISK2:   mov     al,[ebp]                ; get the scan code
        and     eax,127                 ; make it 32 bits long
        test    [byte ptr KeyTable+eax],128
        jz      ISK3                    ; if not pressed, skip this bit...

        mov     al,dh                   ; al = the mask
        xor     al,255                  ; it is an AND mask
        and     [ECX],al                ; clear the appropriate bit
ISK3:
        inc     ebp                     ; look at next scan table entry
        add     dh,dh                   ; shift the mask one to the left
        cmp     dh,20h
        jne     ISK2                    ; loop back until done one column

        inc     ecx                     ; do the next line
        dec     dl                      ; do it 8 times...
        jnz     ISK1

        mov     ebx,offset Specials     ; check the special combinations
ISK4:   call    ICheckSpecial           ; do the check
        or      al,al                   ; if it was used, exit
        jnz     ISK6
ISK5:   mov     al,[ebx]                ; find the next one
        inc     ebx
        or      al,al
        jnz     ISK5
        mov     al,[ebx]                ; reached the end of the specials ?
        or      al,al
        jnz     ISK4                    ; if not try another

ISK6:   pop     ebp
        pop     edx
        pop     ecx
        pop     ebx
        pop     eax
        ret

.data
ScanCodeTable:
        db      kShiftR,kZ,kX,kC,kV  ; 1
        db      kA,kS,kD,kF,kG          ; 2
        db      kQ,kW,kE,kR,kT          ; 3
        db      k1,k2,k3,k4,k5          ; 4
        db      k0,k9,k8,k7,k6          ; 5
        db      kP,kO,kI,kU,kY          ; 6
        db      kEnter,kL,kK,kJ,kH      ; 7
        db      kSpace,kStop,kM,kN,kB   ; 8

Specials:
        db      kBackspace              ; Backspace closes
        db              1,1, 5,1, 0     ; Shift and Zero

        db      kLeft                   ; Left Closes
        db              1,1, 4,16, 0    ; Shift and 5

        db      kUp                     ; Up closes
        db              1,1, 5,8, 0     ; Shift and 7

        db      kDown                   ; Down closes
        db              1,1, 5,16, 0    ; Shift and 6

        db      kRight                  ; Right closes
        db              1,1, 5,4,  0    ; Shift and 8


        db      kSemi+128               ; : closes
        db              1,1, 1,2,  0    ; Shift and Z
        db      kSemi                   ; ; closes
        db              1,1, 1,4,  0    ; Shift and X

        db      kComma                  ; , closes
        db              1,1, 8,2,  0    ; Shift and .

        db      kMinus                  ; - closes
        db              1,1, 7,8,  0    ; Shift and J
        db      kEqual+128              ; + closes
        db              1,1, 7,4,  0    ; Shift and K
        db      kEqual                  ; = closes
        db              1,1, 7,2,  0    ; Shift and L
        db      kQuote                  ; ' closes
        db              1,1, 6,1,  0    ; Shift and P (")

;       db      kLBrk                   ; [ closes
;       db              1,1, 6,4,  0    ; Shift I
;       db      kRBrk                   ; ] closes
;       db              1,1, 6,2,  0    ; Shift O

        db      kShiftL                 ; Left Shift closes
        db              1,1, 0          ; Right Shift

        db      0

        align   dword
.code

EndP IScanKeys

; *****************************************************************************
;
;       Check for special key. EBX points to a scan code (bit 7 requires
;       shift if set). If true, then clear all the relevant bits
;
; *****************************************************************************

ICheckSpecial Proc
        push    ebx
        push    ecx
        mov     al,[ebx]                ; al is the key to be pressed
        and     eax,07Fh                ; make it 7 bit to 32
        test    [byte ptr KeyTable+eax],128
        jz      _ICSX                   ; if not set, exit
        mov     al,[ebx]                ; check if shift is required ?
        test    al,128
        jz      _ICS3                   ; if not continue
        test    [byte ptr KeyTable+kShiftL],128
        jnz     _ICS3
        test    [byte ptr KeyTable+kShiftR],128
        jz      _ICSX

_ICS3:  inc     ebx
_ICS0:  mov     al,[ebx]                ; al is the offset in bittable+1
        or      al,al                   ; zero when hit table
        jz      _ICS2
        and     eax,15                  ; eax = offset
        add     eax,offset BitTable-1   ; it's an address now.
        mov     cl,[ebx+1]              ; bl is the mask.
        xor     cl,255                  ; change to an AND mask
        and     [eax],cl                ; clear the appropriate bits.
        add     ebx,2                   ; anything else need closing ?
        jmp     _ICS0

_ICS2:  mov     eax,-1                  ; passed, return -1
        jmp     _ICS1
_ICSX:  xor     eax,eax                 ; failed return zero
_ICS1:  pop     ecx
        pop     ebx
        ret
EndP ICheckSpecial

; *****************************************************************************
;
;                         Write value AH to port AL
;
; *****************************************************************************

IOutPort Proc
        cmp     al,0FDh                 ; out FD turns off NMI
        je      _NMIOff
        cmp     al,0FEh                 ; out FE turns on NMI
        je      _NMIOn
        cmp     al,0FFh                 ; out FF turns on HSYNC & frees
        je      _HSyncOn                ; the ULA Line Counter
        ret

_NMIOff:mov     [NMIEnabled],0          ; no NMIs (OUT to $FD)
        ret

_NMIOn: mov     [NMIEnabled],-1         ; NMIs generated (OUT to $FE)
        mov     [Cycles],0              ; reset cycle count for Synchronising
        ret

_HSyncOn:                               ; Start HSYNC (Out to $FF)
        mov     [ULALine],0             ; we just reset the ULA Counter
        mov     [ULADelay],1            ; delay because it needs it !
        cmp     [IsZX80Hardware],0      ; not required for the ZX80 ?
        jz      _NotZX80
        mov     [ULADelay],0
_NotZX80:
        mov     al,0                    ; VSYNC off
        call    SCSetSync
        ret                             ; it is bumped by IRQ ?
                                        ; Don't need a real HSYNC for Emulator
EndP IOutPort

end
