; *****************************************************************************
; *****************************************************************************
;
;       File    : Extern.Inc
;       Purpose : External Definitions
;       Created : 24th September 1997
;       Modified: 5th November 1997
;
; *****************************************************************************
; *****************************************************************************

Global Start:Near                       ; Start of main program

VCSetColour = 128                       ; Constants for Video Functions
VCSetBack   = 144
VCSetX      = 160
VCSetY      = 200
VCClear     = 12

SnapSize    = 524288                    ; Size of 'snap' buffer.
SnapMax     = 1024                      ; Max Snaps loadable into menu

Global VSetGraphicMode:Proc             ; Set 320x200 VGA Mode
Global VSetTextMode:Proc                ; Set 80x25 Text Mode
Global VPrint:Proc                      ; Print a character in AL
Global VPrintByte:Proc                  ; Print a byte in AL
Global VPrintWord:Proc                  ; Print a word in AX
Global VPrintString:Proc                ; Print a ASCIIZ String at EBX

Global LoadTapePatch:Proc
Global SaveTapePatch:Proc
Global SaveTapePatch2:Proc

Global CMHandler:Proc                   ; Handle command line (called first)
Global CMFileName:Proc                  ; Get a file name with given ext.

Global FLoad:Proc                       ; Load a File
Global FSaveMem:Proc
Global FDirectory:Proc                  ; Do a Snapshot Directory
Global FGetName:Proc

Global KInitialise:Proc                 ; Set up Keyboard Handler
Global KTerminate:Proc                  ; Replace old Keyboard Handler
Global KTest:Proc                       ; Test Keyboard Handler
Global KeyTable:Near                    ; Keyboard Table
Global KGet:Proc                        ; Get Keystroke from given list

Global Menu:Proc                        ; Select from menu
Global MenuLoad:Proc                    ; Select file to load from menu

Global SCRebuild:Proc                   ; Redraw the whole screen
Global SCInitialise:Proc                ; Initialise the screen
Global SCFrame:Proc
Global SCFastPaint:Proc
Global SCExecDisplayFile:Proc

Global ResetBackground:Proc
     
Global DReset:Proc                      ; Reset the Computer

Global BDisplay:Proc                    ; Display the Debug Info Screen
Global BDebug:Proc                      ; Enter the Debugger
Global BSetExecute:Proc                 ; Set the Execution Mode

dbmRun = 0                              ; Debug Mode - run normally
dbmSingleStep = 1                       ; run a single instruction
dbmRunBreak = 2                         ; run until breakpoint reached
dbmStepOver = 3                         ; run until Stack Pointer Breakpoint

Global Disassemble:Near                 ; Table of Disassembly Information

Global BError:Near                      ; Go here on error.
Global XEcute:Proc                      ; Execute a single instruction
Global XEcuteAlways:Proc                ; Execute, no HiRAM Test
Global GoDOS:Near
Global HaltCheck:Proc

Global _OG0_FF:Near                     ; RST 38h
Global _OG0_F3:Near                     ; DI
Global _OG0_FB:Near                     ; EI
Global _OG0_E9:Near                     ; JP (HL)
Global _OG2_45:Near                     ; RETN
Global _OG2_FF:Near                     ; Interrupt (Mode 2)
Global _OG2_FE:Near                     ; Non maskable Interrupt
Global _OG0_08:Near                     ; EX AF,AF'

Global IInPort:Proc                     ; Port I/O Commands
Global IOutPort:Proc
Global IScanKeys:Proc                   ; Scan Speccy Keys

Global CNMenu:Proc                      ; Configuration Menu
Global CheckInterrupt:Proc              ; Check Interrupt
Global ExecDisplayFile:Proc             ; Execute with PC >= $8000
Global VSync:Proc                       ; VSYNC Code

Global Memory:Near                      ; Address of 64k memory block
Global FileName:DWord                   ; Address of current filename (or 0)
Global DbgData:Word                     ; Debugger Data Pointer
Global Cycles:DWord                     ; No of Cycles in this scan line
Global Break:Word                       ; Breakpoint
Global FrameCount:DWord                 ; # of frames rendered
Global FrmSkip:DWord                    ; Frame Skipping Rate
Global FrmRate:DWord                    ; Frames per clock tick rate
Global ExecPtr:DWord                    ; Execute one command vector
Global NMIEnabled:DWord                 ; Non-Zero if NMI Enabled
Global NMICounter:DWord                 ; Count number of NMI periods.
Global ULALine:DWord                    ; ULA Line Counter
Global VideoLine:DWord                  ; Video line counter
Global Redraw:DWord                     ; Redraw when zero.
Global ESSeg:Word                       ; ES Value
Global Scheme:DWord
Global Background:DWord
Global Foreground:DWord
Global FSnapList:Near
Global FSnapCount:DWord
Global DStartData:Near
Global DEndData:Near
Global FastPaint:Near

Global _AF:Near                         ; Z80 Register Definitions
Global _BC:Near
Global _DE:Near
Global _HL:Near
Global _A:Byte
Global _F:Byte
Global _B:Byte
Global _C:Byte
Global _D:Byte
Global _E:Byte
Global _H:Byte
Global _L:Byte
Global _IX:Word
Global _IY:Word
Global _SP:Word
Global _PC:Word
Global _XAF:Word
Global _XBC:Word
Global _XDE:Word
Global _XHL:Word
Global _IMode:Byte                      ; Interrupt Mode 0-2
Global _IFF1:Byte                       ; Interrupt Flags
Global _IFF2:Byte
Global _I:Byte
Global _R:Byte
Global _RBit7:Byte

kEsc    = 1                             ; Keyboard Scan Codes
k0      = 0Bh                           ; Number keys 0-9
k1      = 02h
k2      = 03h
k3      = 04h
k4      = 05h
k5      = 06h
k6      = 07h
k7      = 08h
k8      = 09h
k9      = 0Ah
kA      = 1Eh                           ; Letters A-F
kB      = 30h
kC      = 2Eh
kD      = 20h
kE      = 12h
kF      = 21h

kG      = 22h                           ; Letters G-Z
kH      = 23h
kI      = 17h
kJ      = 24h
kK      = 25h
kL      = 26h
kM      = 32H
kN      = 31h
kO      = 18h
kP      = 19h
kQ      = 10h
kR      = 13h
kS      = 1Fh
kT      = 14h
kU      = 16h
kV      = 2Fh
kW      = 11h
kX      = 2Dh
kY      = 15h
kZ      = 2Ch


kCtrl   = 1Dh                           ; control keys
kShiftL = 2Ah
kShiftR = 36h

KAlt    = 38h
kF1     = 3Bh                           ; function keys
kF2     = 3Ch
kF3     = 3Dh
kF4     = 3Eh
kF5     = 3Fh
kF6     = 40h
kF7     = 41h
kF8     = 42h
kF9     = 43h
kF10    = 44h

kSpace  = 39h                           ; space bar
kEnter  = 1Ch                           ; return key

kBackSpace = 0Eh                        ; backspace key
kUp     = 048h                          ; arrow keys
kDown   = 050h
kLeft   = 04Bh
kRight  = 04Dh
kPageUp = 049h                          ; Control keys
kPageDn = 051h
kHome   = 047h
kEnd    = 04Fh

kSemi   = 27h                           ; Punctuation Keys
kEqual  = 0Dh
kStop   = 34h
kMinus  = 0Ch
kComma  = 33h
kQuote =  28h
kXPlus =  4Eh
kVBar =   29h                           ; wierd one, top left
